package de.hft.stuttgart.citydoctor2.zip;

import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import de.hft.stuttgart.citydoctor2.utils.ArchivePacker;
import de.hft.stuttgart.citydoctor2.utils.Localization;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.Serial;
import java.io.Serializable;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

/**
 * This class serves as a representation of a ZIP-archive containing CityGml files. The contained files are represented
 * by {@link CityGmlZipEntry} objects.
 */
public class CityGmlZipArchive implements Serializable {

    private static final Logger logger = LogManager.getLogger(CityGmlZipArchive.class);

    @Serial
    private static final long serialVersionUID = 2168389511043362615L;

    private List<CityGmlZipEntry> entries;
    private final Path archivePath;
    private final String archiveNameRE;

    /**
     * Registers a ZIP-file as an archive. All contained XML and GML files will be registered as {@link CityGmlZipEntry} objects.
     *
     * @param zipFile path of the ZIP-file
     * @return A {@link CityGmlZipArchive} object representing the archive, or null if read-access for the ZIP-file was denied
     * @throws MalformedZipFileException if the ZIP-file has encoding errors
     */
    public static CityGmlZipArchive register(String zipFile) throws MalformedZipFileException {
        ArrayList<CityGmlZipEntry> archiveEntries = new ArrayList<>();
        CityGmlZipArchive cgmlArchive = new CityGmlZipArchive(Path.of(zipFile));
        try (ZipInputStream zis = new ZipInputStream(new FileInputStream(zipFile))) {
            ZipEntry ze;
            while ((ze = zis.getNextEntry()) != null) {
                if (ze.isDirectory()) {
                    continue;
                }
                if (ze.getName().endsWith(".gml") || ze.getName().endsWith(".xml")) {
                    // Ignore report files generated by CityDoctor
                    if (ze.getName().endsWith("_report.xml")) {
                        continue;
                    }
                    archiveEntries.add(CityGmlZipEntry.register(ze, cgmlArchive));
                }
            }
            cgmlArchive.setEntries(archiveEntries);
            return cgmlArchive;
        } catch (IOException e) {
            logger.error(Localization.getText("CityGmlZipArchive.ioError"));
            logger.debug(e.getMessage(), e);
            return null;
        } catch (IllegalArgumentException e) {
            logger.error(Localization.getText("CityGmlZipArchive.malformedZipFile"));
            throw new MalformedZipFileException(Localization.getText("CityGmlZipArchive.malformedZipFile"));
        } catch (Exception e) {
            logger.error(Localization.getText("CityGmlZipArchive.generalException"));
            logger.debug(e.getMessage(), e);
            throw (e);
        }
    }

    /**
     * Attempts to load all contained {@link CityGmlZipEntry} objects into memory.
     *
     * @param config Parser configuration for the entries
     */
    public void mountArchive(ParserConfiguration config) {
        try (ZipFile zip = new ZipFile(archivePath.toFile())) {
            for (CityGmlZipEntry entry : entries) {
                entry.loadEntry(config);
            }
        } catch (IOException e) {
            logger.error(e);
        }
    }

    private CityGmlZipArchive(Path archivePath) {
        this.archivePath = archivePath;
        this.archiveNameRE = archivePath.getFileName().toString().replace(".zip", "") + File.separator;
    }

    private void setEntries(List<CityGmlZipEntry> entries) {
        this.entries = entries;
    }

    /**
     * Saves this archive as a ZIP-file.
     * @param path target path for the export
     */
    public void exportToZipFile(String path) {
        ArchivePacker.packArchive(path, this);
    }

    /**
     * Gets an entry in this archive by its subpath.
     *
     * @param subpath subpath to the entry from this archive's root directory
     * @return the corresponding entry, or null if entry could not be found
     */
    public CityGmlZipEntry getEntry(String subpath) {
        subpath = stripArchivePath(subpath);
        for (CityGmlZipEntry entry : entries) {
            String entryName = stripArchivePath(entry.getEntrySubPath());
            if (entryName.equals(subpath)) {
                return entry;
            }
        }
        return null;
    }

    /**
     * Strips the root directory name of this archive from an entry subpath.
     * @param path the relative path to the entry
     * @return String of the path without the root directory
     */
    private String stripArchivePath(String path) {
        Path systemPath = Path.of(path);
        return systemPath.toString().replace(archiveNameRE, "");
    }

    /**
     * Gets the path to the ZIP-file associated with this archive.
     * @return path of the ZIP-file
     */
    public Path getArchivePath() {
        return archivePath;
    }

    /**
     * Gets the entry list of this archive.
     * @return the entry list
     */
    public List<CityGmlZipEntry> getEntries() {
        return entries;
    }

}
