package de.hft.stuttgart.citydoctor2.zip;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

/**
 * This class serves as a resource for read-access to the file that is being represented by
 * a CityGmlZipEntry object.
 * <p/>
 * Does not decompress the file or the parent ZIP-archive.
 */
public class CityGmlZipEntryFile implements AutoCloseable {

    private final ZipFile zip;
    private final ZipEntry zipEntry;
    private final CityGmlZipEntry cgmlZipEntry;
    private boolean closed = false;

    /**
     * Creates a {@link CityGmlZipEntryFile} for a {@link CityGmlZipEntry} object's referenced file.
     *
     * @param entry the requested entry
     * @throws IOException if read access to the ZIP-archive is denied.
     */
    public CityGmlZipEntryFile(CityGmlZipEntry entry) throws IOException {
        CityGmlZipArchive archive = entry.getArchive();
        zip = new ZipFile(archive.getArchivePath().toFile());
        zipEntry = zip.getEntry(entry.getEntrySubPath());
        this.cgmlZipEntry = entry;
    }

    /**
     * Gets a new InputStream of this file.
     * <p/>
     * Returned streams are independent of each other, but will all be closed when the {@link CityGmlZipEntryFile} object
     * is closed.
     * @return an InputStream of this file.
     * @throws IOException when this method is called after the file has been closed.
     */
    public InputStream getInputStream() throws IOException {
        if (closed){
            throw new IOException("CityGmlZipEntryFile closed");
        }
        return zip.getInputStream(zipEntry);
    }

    /**
     * Gets an estimation of this file's filesize.
     * @return filesize in bytes, -1L if unknown
     * @throws IOException when this method is called after the file has been closed.
     */
    public long getFileSize() throws IOException {
        if (closed){
            throw new IOException("CityGmlZipEntryFile closed");
        }
        if (cgmlZipEntry.getFileSize() == -1L) {
            if (zipEntry.getSize() == -1) {
                long bytes = 0;
                InputStream is = new BufferedInputStream(this.getInputStream());
                for (int i = is.read(); i != -1; i = is.read()) {
                    bytes++;
                }
                cgmlZipEntry.setFileSize(bytes);
            } else {
                cgmlZipEntry.setFileSize(zipEntry.getSize());
            }
        }
        return cgmlZipEntry.getFileSize();
    }

    @Override
    public void close() throws IOException {
        zip.close();
        closed = true;
    }
}
