/*-
 *  Copyright 2020 Beuth Hochschule für Technik Berlin, Hochschule für Technik Stuttgart
 * 
 *  This file is part of CityDoctor2.
 *
 *  CityDoctor2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CityDoctor2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with CityDoctor2.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.hft.stuttgart.citydoctor2.datastructure;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.util.ArrayList;
import java.util.List;

import org.citygml4j.core.model.deprecated.core.DeprecatedPropertiesOfAbstractCityObject;
import org.citygml4j.core.model.deprecated.core.DeprecatedPropertiesOfAbstractThematicSurface;
import org.citygml4j.core.model.deprecated.transportation.DeprecatedPropertiesOfAbstractTransportationSpace;
import org.citygml4j.core.model.transportation.Intersection;
import org.citygml4j.core.model.transportation.Railway;
import org.citygml4j.core.model.transportation.Road;
import org.citygml4j.core.model.transportation.Section;
import org.citygml4j.core.model.transportation.Square;
import org.citygml4j.core.model.transportation.Track;
import org.citygml4j.core.model.transportation.TrafficArea;
import org.citygml4j.core.model.transportation.TrafficSpace;
import org.citygml4j.core.model.transportation.Waterway;
import org.citygml4j.core.util.geometry.GeometryFactory;
import org.junit.Test;
import org.mockito.Mockito;
import org.xmlobjects.gml.model.geometry.aggregates.MultiSurfaceProperty;

import de.hft.stuttgart.citydoctor2.check.CheckError;
import de.hft.stuttgart.citydoctor2.check.CheckId;
import de.hft.stuttgart.citydoctor2.check.CheckResult;
import de.hft.stuttgart.citydoctor2.check.ResultStatus;
import de.hft.stuttgart.citydoctor2.datastructure.TransportationObject.TransportationType;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;

public class TransportationObjectTest {

	@Test
	public void testGetFeatureType() {
		TopLevelTransportFeature to = TopLevelTransportFeature.from(Mockito.mock(Road.class));
		assertEquals(FeatureType.TRANSPORTATION, to.getFeatureType());
	}

	@Test
	public void testObjectConstruction() {
		TopLevelTransportFeature road = TopLevelTransportFeature.from(Mockito.mock(Road.class));
		assertEquals(TransportationType.ROAD, road.getTransportationType());
		TopLevelTransportFeature track = TopLevelTransportFeature.from(Mockito.mock(Track.class));
		assertEquals(TransportationType.TRACK, track.getTransportationType());
		TopLevelTransportFeature waterway = TopLevelTransportFeature.from(Mockito.mock(Waterway.class));
		assertEquals(TransportationType.WATERWAY, waterway.getTransportationType());
		TopLevelTransportFeature railway = TopLevelTransportFeature.from(Mockito.mock(Railway.class));
		assertEquals(TransportationType.RAILWAY, railway.getTransportationType());
		TopLevelTransportFeature square = TopLevelTransportFeature.from(Mockito.mock(Square.class));
		assertEquals(TransportationType.SQUARE, square.getTransportationType());
	}
	
	@Test(expected = IllegalStateException.class)
	public void testReCreateGeometriesSolid() {
		TopLevelTransportFeature to = TopLevelTransportFeature.from(Mockito.mock(Road.class));
		to.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.SOLID, Lod.LOD1));
		GeometryFactory factory = GeometryFactory.newInstance();
		to.reCreateGeometries(factory, mock(ParserConfiguration.class));
	}


	@Test
	public void testReCreateGeometriesTransportationSpace() {
		Road roadMock = mock(Road.class);
		TopLevelTransportFeature top = TopLevelTransportFeature.from(roadMock);
		top.setGmlObject(roadMock);
		TransportSection sectionMock = Mockito.mock(TransportSection.class);
		TransportSection intersectionMock = Mockito.mock(TransportSection.class);
		TrafficSpaceObject tsMock = Mockito.mock(TrafficSpaceObject.class);
		TrafficSpaceObject auxTsMock = Mockito.mock(TrafficSpaceObject.class);

		top.addSection(sectionMock);
		top.addIntersection(intersectionMock);
		top.addTrafficSpace(tsMock);
		top.addAuxTrafficSpace(auxTsMock);

		top.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD2));
		GeometryFactory factory = GeometryFactory.newInstance();
		top.reCreateGeometries(factory, mock(ParserConfiguration.class));

		verify(roadMock).setLod2MultiSurface(any());
		verify(sectionMock).reCreateGeometries(any(), any());
		verify(intersectionMock).reCreateGeometries(any(), any());
		verify(tsMock).reCreateGeometries(any(), any());
		verify(auxTsMock).reCreateGeometries(any(), any());
	}

	@Test
	public void testReCreateGeometriesTrafficSpace() {
		TrafficSpaceObject tso = new TrafficSpaceObject(TrafficSpaceObject.TrafficSpaceType.TRAFFIC_SPACE);
		TrafficSpace tsMock = mock(TrafficSpace.class);
		tso.setGmlObject(tsMock);
		TrafficAreaObject taoMock = Mockito.mock(TrafficAreaObject.class);

		tso.addTrafficArea(taoMock);

		tso.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD2));
		GeometryFactory factory = GeometryFactory.newInstance();
		tso.reCreateGeometries(factory, mock(ParserConfiguration.class));

		verify(tsMock).setLod2MultiSurface(any());
		verify(taoMock).reCreateGeometries(any(), any());
	}

	@Test
	public void testReCreateGeometriesTrafficArea() {
		TrafficAreaObject tao = new TrafficAreaObject(TrafficAreaObject.TrafficAreaType.TRAFFIC_AREA);
		TrafficArea taMock = mock(TrafficArea.class);
		tao.setGmlObject(taMock);

		tao.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD2));
		GeometryFactory factory = GeometryFactory.newInstance();
		tao.reCreateGeometries(factory, mock(ParserConfiguration.class));

		verify(taMock).setLod2MultiSurface(any());
	}

	@Test
	public void testReCreateGeometriesMultiSurfaceLod0() {
		Road roadMock = mock(Road.class);
		TopLevelTransportFeature top = TopLevelTransportFeature.from(roadMock);
		top.setGmlObject(roadMock);

		TrafficSpaceObject ts = new TrafficSpaceObject(TrafficSpaceObject.TrafficSpaceType.TRAFFIC_SPACE);
		TrafficSpace trafficSpaceMock = mock(TrafficSpace.class);
		ts.setGmlObject(trafficSpaceMock);

		TrafficAreaObject ta = new TrafficAreaObject(TrafficAreaObject.TrafficAreaType.TRAFFIC_AREA);
		TrafficArea trafficAreaMock = mock(TrafficArea.class);
		ta.setGmlObject(trafficAreaMock);

		top.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD0));
		ts.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD0));
		ta.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD0));

		GeometryFactory factory = GeometryFactory.newInstance();
		top.reCreateGeometries(factory, mock(ParserConfiguration.class));
		ts.reCreateGeometries(factory, mock(ParserConfiguration.class));
		ta.reCreateGeometries(factory, mock(ParserConfiguration.class));

		verify(roadMock).setLod0MultiSurface(any());
		verify(trafficSpaceMock).setLod0MultiSurface(any());
		verify(trafficAreaMock).setLod0MultiSurface(any());
	}
	
	@Test
	public void testReCreateGeometriesMultiSurfaceLod1() {
		Road roadMock = mock(Road.class);
		TopLevelTransportFeature top = TopLevelTransportFeature.from(roadMock);
		top.setGmlObject(roadMock);

		TrafficAreaObject ta = new TrafficAreaObject(TrafficAreaObject.TrafficAreaType.TRAFFIC_AREA);
		TrafficArea trafficAreaMock = mock(TrafficArea.class);
		ta.setGmlObject(trafficAreaMock);

		DeprecatedPropertiesOfAbstractTransportationSpace dSpace = mock(DeprecatedPropertiesOfAbstractTransportationSpace.class);
		when(roadMock.getDeprecatedProperties()).thenReturn(dSpace);

		try {
			top.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD1));
		} catch (IllegalArgumentException e) {
			fail("Casting of original GML object to AbstractTransportationSpace should work");
		}

		ta.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD1));
		GeometryFactory factory = GeometryFactory.newInstance();
		top.reCreateGeometries(factory, mock(ParserConfiguration.class));
		ta.reCreateGeometries(factory, mock(ParserConfiguration.class));

		verify(dSpace).setLod1MultiSurface(any());
		verify(trafficAreaMock).setLod1MultiSurface(any());
	}

	
	@Test
	public void testReCreateGeometriesMultiSurfaceLod2() {
		Road roadMock = mock(Road.class);
		TopLevelTransportFeature top = TopLevelTransportFeature.from(roadMock);
		top.setGmlObject(roadMock);

		TrafficSpaceObject ts = new TrafficSpaceObject(TrafficSpaceObject.TrafficSpaceType.TRAFFIC_SPACE);
		TrafficSpace trafficSpaceMock = mock(TrafficSpace.class);
		ts.setGmlObject(trafficSpaceMock);

		TrafficAreaObject ta = new TrafficAreaObject(TrafficAreaObject.TrafficAreaType.TRAFFIC_AREA);
		TrafficArea trafficAreaMock = mock(TrafficArea.class);
		ta.setGmlObject(trafficAreaMock);

		top.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD2));
		ts.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD2));
		ta.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD2));

		GeometryFactory factory = GeometryFactory.newInstance();
		top.reCreateGeometries(factory, mock(ParserConfiguration.class));
		ts.reCreateGeometries(factory, mock(ParserConfiguration.class));
		ta.reCreateGeometries(factory, mock(ParserConfiguration.class));

		verify(roadMock).setLod2MultiSurface(any());
		verify(trafficSpaceMock).setLod2MultiSurface(any());
		verify(trafficAreaMock).setLod2MultiSurface(any());
	}
	
	@Test
	public void testReCreateGeometriesMultiSurfaceLod3() {
		Road roadMock = mock(Road.class);
		TopLevelTransportFeature top = TopLevelTransportFeature.from(roadMock);
		top.setGmlObject(roadMock);

		TrafficSpaceObject ts = new TrafficSpaceObject(TrafficSpaceObject.TrafficSpaceType.TRAFFIC_SPACE);
		TrafficSpace trafficSpaceMock = mock(TrafficSpace.class);
		ts.setGmlObject(trafficSpaceMock);

		TrafficAreaObject ta = new TrafficAreaObject(TrafficAreaObject.TrafficAreaType.TRAFFIC_AREA);
		TrafficArea trafficAreaMock = mock(TrafficArea.class);
		ta.setGmlObject(trafficAreaMock);

		top.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD3));
		ts.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD3));
		ta.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD3));

		GeometryFactory factory = GeometryFactory.newInstance();
		top.reCreateGeometries(factory, mock(ParserConfiguration.class));
		ts.reCreateGeometries(factory, mock(ParserConfiguration.class));
		ta.reCreateGeometries(factory, mock(ParserConfiguration.class));

		verify(roadMock).setLod3MultiSurface(any());
		verify(trafficSpaceMock).setLod3MultiSurface(any());
		verify(trafficAreaMock).setLod3MultiSurface(any());
	}
	
	@Test
	public void testReCreateGeometriesMultiSurfaceLod4() {
		Road roadMock = mock(Road.class);
		TopLevelTransportFeature top = TopLevelTransportFeature.from(roadMock);
		top.setGmlObject(roadMock);

		TrafficAreaObject ta = new TrafficAreaObject(TrafficAreaObject.TrafficAreaType.TRAFFIC_AREA);
		TrafficArea trafficAreaMock = mock(TrafficArea.class);
		ta.setGmlObject(trafficAreaMock);

		DeprecatedPropertiesOfAbstractTransportationSpace dSpace = mock(DeprecatedPropertiesOfAbstractTransportationSpace.class);
		when(roadMock.getDeprecatedProperties()).thenReturn(dSpace);
		DeprecatedPropertiesOfAbstractThematicSurface dArea = mock(DeprecatedPropertiesOfAbstractThematicSurface.class);
		when(trafficAreaMock.getDeprecatedProperties()).thenReturn(dArea);

		ta.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD4));
		try {
			top.addGeometry(GeometryTestUtils.createDummyGeometry(GeometryType.MULTI_SURFACE, Lod.LOD4));
		} catch (IllegalArgumentException e) {
			fail("Casting of original GML object to AbstractTransportationSpace should work");
		}

		GeometryFactory factory = GeometryFactory.newInstance();
		top.reCreateGeometries(factory, mock(ParserConfiguration.class));
		ta.reCreateGeometries(factory, mock(ParserConfiguration.class));

		verify(dSpace).setLod4MultiSurface(any());
		verify(dArea).setLod4MultiSurface(any());
	}


	@Test
	public void testContainsErrorTransportationSpace() {
		Road roadMock = mock(Road.class);
		TopLevelTransportFeature top = TopLevelTransportFeature.from(roadMock);
		top.setGmlObject(roadMock);
		TransportSection sectionMock = Mockito.mock(TransportSection.class);
		TransportSection intersectionMock = Mockito.mock(TransportSection.class);
		TrafficSpaceObject tsMock = Mockito.mock(TrafficSpaceObject.class);
		TrafficSpaceObject auxTsMock = Mockito.mock(TrafficSpaceObject.class);

		top.addSection(sectionMock);
		top.addIntersection(intersectionMock);
		top.addTrafficSpace(tsMock);
		top.addAuxTrafficSpace(auxTsMock);

		top.addCheckResult(new CheckResult(CheckId.C_GE_P_HOLE_OUTSIDE, ResultStatus.ERROR, mock(CheckError.class)));
		assertTrue(top.containsError(CheckId.C_GE_P_HOLE_OUTSIDE));
		assertFalse(top.containsError(CheckId.C_GE_P_INNER_RINGS_NESTED));

		verify(sectionMock).containsError(any());
		verify(intersectionMock).containsError(any());
		verify(tsMock).containsError(any());
		verify(auxTsMock).containsError(any());
	}

	@Test
	public void testContainsErrorTrafficSpace() {
		TrafficSpaceObject tso = new TrafficSpaceObject(TrafficSpaceObject.TrafficSpaceType.TRAFFIC_SPACE);
		TrafficSpace tsMock = mock(TrafficSpace.class);
		tso.setGmlObject(tsMock);
		TrafficAreaObject taMock = mock(TrafficAreaObject.class);

		tso.addTrafficArea(taMock);

		tso.addCheckResult(new CheckResult(CheckId.C_GE_P_HOLE_OUTSIDE, ResultStatus.ERROR, mock(CheckError.class)));
		assertTrue(tso.containsError(CheckId.C_GE_P_HOLE_OUTSIDE));
		assertFalse(tso.containsError(CheckId.C_GE_P_INNER_RINGS_NESTED));

		verify(taMock).containsError(any());
	}

	@Test
	public void testContainsErrorTrafficArea() {
		TrafficAreaObject tao = new TrafficAreaObject(TrafficAreaObject.TrafficAreaType.TRAFFIC_AREA);
		TrafficArea taMock = mock(TrafficArea.class);
		tao.setGmlObject(taMock);

		tao.addCheckResult(new CheckResult(CheckId.C_GE_P_HOLE_OUTSIDE, ResultStatus.ERROR, mock(CheckError.class)));
		assertTrue(tao.containsError(CheckId.C_GE_P_HOLE_OUTSIDE));
		assertFalse(tao.containsError(CheckId.C_GE_P_INNER_RINGS_NESTED));
	}

	@Test
	public void testClearAllContainedCheckResultsTransportationSpace() {
		Road roadMock = mock(Road.class);
		TopLevelTransportFeature top = TopLevelTransportFeature.from(roadMock);
		top.setGmlObject(roadMock);
		TransportSection sectionMock = Mockito.mock(TransportSection.class);
		TransportSection intersectionMock = Mockito.mock(TransportSection.class);
		TrafficSpaceObject tsMock = Mockito.mock(TrafficSpaceObject.class);
		TrafficSpaceObject auxTsMock = Mockito.mock(TrafficSpaceObject.class);

		top.addSection(sectionMock);
		top.addIntersection(intersectionMock);
		top.addTrafficSpace(tsMock);
		top.addAuxTrafficSpace(auxTsMock);

		assertFalse(top.containsError(CheckId.C_GE_P_HOLE_OUTSIDE));
		top.addCheckResult(new CheckResult(CheckId.C_GE_P_HOLE_OUTSIDE, ResultStatus.ERROR, mock(CheckError.class)));
		assertTrue(top.containsError(CheckId.C_GE_P_HOLE_OUTSIDE));
		top.clearAllContainedCheckResults();
		assertFalse(top.containsError(CheckId.C_GE_P_HOLE_OUTSIDE));

		verify(sectionMock).clearAllContainedCheckResults();
		verify(intersectionMock).clearAllContainedCheckResults();
		verify(tsMock).clearAllContainedCheckResults();
		verify(auxTsMock).clearAllContainedCheckResults();
	}

	@Test
	public void testClearAllContainedCheckResultsTrafficSpace() {
		TrafficSpaceObject tso = new TrafficSpaceObject(TrafficSpaceObject.TrafficSpaceType.TRAFFIC_SPACE);
		TrafficSpace tsMock = mock(TrafficSpace.class);
		tso.setGmlObject(tsMock);
		TrafficAreaObject taMock = mock(TrafficAreaObject.class);

		tso.addTrafficArea(taMock);

		assertFalse(tso.containsError(CheckId.C_GE_P_HOLE_OUTSIDE));
		tso.addCheckResult(new CheckResult(CheckId.C_GE_P_HOLE_OUTSIDE, ResultStatus.ERROR, mock(CheckError.class)));
		assertTrue(tso.containsError(CheckId.C_GE_P_HOLE_OUTSIDE));
		tso.clearAllContainedCheckResults();
		assertFalse(tso.containsError(CheckId.C_GE_P_HOLE_OUTSIDE));

		verify(taMock).clearAllContainedCheckResults();
	}

	@Test
	public void testClearAllContainedCheckResultsTrafficArea() {
		TrafficAreaObject tao = new TrafficAreaObject(TrafficAreaObject.TrafficAreaType.TRAFFIC_AREA);
		TrafficArea taMock = mock(TrafficArea.class);
		tao.setGmlObject(taMock);

		assertFalse(tao.containsError(CheckId.C_GE_P_HOLE_OUTSIDE));
		tao.addCheckResult(new CheckResult(CheckId.C_GE_P_HOLE_OUTSIDE, ResultStatus.ERROR, mock(CheckError.class)));
		assertTrue(tao.containsError(CheckId.C_GE_P_HOLE_OUTSIDE));
		tao.clearAllContainedCheckResults();
		assertFalse(tao.containsError(CheckId.C_GE_P_HOLE_OUTSIDE));

	}

	@Test
	public void testCollectContainedErrorsTransportationSpace() {
		Road roadMock = mock(Road.class);
		TopLevelTransportFeature top = TopLevelTransportFeature.from(roadMock);
		top.setGmlObject(roadMock);
		TransportSection sectionMock = Mockito.mock(TransportSection.class);
		TransportSection intersectionMock = Mockito.mock(TransportSection.class);
		TrafficSpaceObject tsMock = Mockito.mock(TrafficSpaceObject.class);
		TrafficSpaceObject auxTsMock = Mockito.mock(TrafficSpaceObject.class);

		top.addSection(sectionMock);
		top.addIntersection(intersectionMock);
		top.addTrafficSpace(tsMock);
		top.addAuxTrafficSpace(auxTsMock);

		top.addCheckResult(new CheckResult(CheckId.C_GE_P_HOLE_OUTSIDE, ResultStatus.ERROR, mock(CheckError.class)));
		List<CheckError> errors = new ArrayList<>();
		top.collectContainedErrors(errors);
		assertEquals(1, errors.size());

		verify(sectionMock).collectContainedErrors(errors);
		verify(intersectionMock).collectContainedErrors(errors);
		verify(tsMock).collectContainedErrors(errors);
		verify(auxTsMock).collectContainedErrors(errors);
	}

	@Test
	public void testCollectContainedErrorsTrafficSpace() {
		TrafficSpaceObject tso = new TrafficSpaceObject(TrafficSpaceObject.TrafficSpaceType.TRAFFIC_SPACE);
		TrafficSpace tsMock = mock(TrafficSpace.class);
		tso.setGmlObject(tsMock);
		TrafficAreaObject taMock = mock(TrafficAreaObject.class);

		tso.addTrafficArea(taMock);

		tso.addCheckResult(new CheckResult(CheckId.C_GE_P_HOLE_OUTSIDE, ResultStatus.ERROR, mock(CheckError.class)));
		List<CheckError> errors = new ArrayList<>();
		tso.collectContainedErrors(errors);
		assertEquals(1, errors.size());

		verify(taMock).collectContainedErrors(errors);
	}

	@Test
	public void testCollectContainedErrorsTrafficArea() {
		TrafficAreaObject tao = new TrafficAreaObject(TrafficAreaObject.TrafficAreaType.TRAFFIC_AREA);
		TrafficArea taMock = mock(TrafficArea.class);
		tao.setGmlObject(taMock);

		tao.addCheckResult(new CheckResult(CheckId.C_GE_P_HOLE_OUTSIDE, ResultStatus.ERROR, mock(CheckError.class)));
		List<CheckError> errors = new ArrayList<>();
		tao.collectContainedErrors(errors);
		assertEquals(1, errors.size());
	}

	@Test
	public void testUnsetGmlGeometriesTransportationSpace() {
		Road r = mock(Road.class);
		TopLevelTransportFeature top = TopLevelTransportFeature.from(r);
		top.setGmlObject(r);
		TransportSection mockSection = mock(TransportSection.class);
		TransportSection mockInterestSection = mock(TransportSection.class);
		TrafficSpaceObject mockSpace = mock(TrafficSpaceObject.class);
		TrafficSpaceObject auxMockSpace = mock(TrafficSpaceObject.class);

		top.addSection(mockSection);
		top.addIntersection(mockInterestSection);
		top.addTrafficSpace(mockSpace);
		top.addAuxTrafficSpace(auxMockSpace);

		DeprecatedPropertiesOfAbstractTransportationSpace dSpace = mock(DeprecatedPropertiesOfAbstractTransportationSpace.class);
		when(r.getDeprecatedProperties()).thenReturn(dSpace);

		dSpace.setLod1MultiSurface(mock(MultiSurfaceProperty.class));
		r.setLod2MultiSurface(mock(MultiSurfaceProperty.class));
		r.setLod3MultiSurface(mock(MultiSurfaceProperty.class));
		dSpace.setLod4MultiSurface(mock(MultiSurfaceProperty.class));

		top.unsetGmlGeometries();
		assertNull(dSpace.getLod1MultiSurface());
		assertNull(r.getLod2MultiSurface());
		assertNull(r.getLod3MultiSurface());
		assertNull(dSpace.getLod4MultiSurface());

		verify(mockSection).unsetGmlGeometries();
		verify(mockInterestSection).unsetGmlGeometries();
		verify(mockSpace).unsetGmlGeometries();
		verify(auxMockSpace).unsetGmlGeometries();

	}

	@Test
	public void testUnsetGmlGeometriesTrafficSpace() {
		TrafficSpaceObject tso = new TrafficSpaceObject(TrafficSpaceObject.TrafficSpaceType.TRAFFIC_SPACE);
		TrafficSpace t = mock(TrafficSpace.class);
		tso.setGmlObject(t);
		TrafficAreaObject mockArea = mock(TrafficAreaObject.class);

		tso.addTrafficArea(mockArea);

		DeprecatedPropertiesOfAbstractCityObject dSpace = mock(DeprecatedPropertiesOfAbstractCityObject.class);
		when(t.getDeprecatedProperties()).thenReturn(dSpace);

		t.setLod0MultiSurface(mock(MultiSurfaceProperty.class));
		t.setLod2MultiSurface(mock(MultiSurfaceProperty.class));
		t.setLod3MultiSurface(mock(MultiSurfaceProperty.class));

		tso.unsetGmlGeometries();
		assertNull(t.getLod0MultiSurface());
		assertNull(t.getLod2MultiSurface());
		assertNull(t.getLod3MultiSurface());

		verify(mockArea).unsetGmlGeometries();

	}

	@Test
	public void testUnsetGmlGeometriesTrafficArea() {
		TrafficAreaObject ta = new TrafficAreaObject(TrafficAreaObject.TrafficAreaType.TRAFFIC_AREA);
		TrafficArea r = mock(TrafficArea.class);
		ta.setGmlObject(r);
		DeprecatedPropertiesOfAbstractThematicSurface dSpace = mock(DeprecatedPropertiesOfAbstractThematicSurface.class);
		when(r.getDeprecatedProperties()).thenReturn(dSpace);

		r.setLod0MultiSurface(mock(MultiSurfaceProperty.class));
		r.setLod1MultiSurface(mock(MultiSurfaceProperty.class));
		r.setLod2MultiSurface(mock(MultiSurfaceProperty.class));
		r.setLod3MultiSurface(mock(MultiSurfaceProperty.class));
		dSpace.setLod4MultiSurface(mock(MultiSurfaceProperty.class));

		ta.unsetGmlGeometries();
		assertNull(r.getLod0MultiSurface());
		assertNull(r.getLod1MultiSurface());
		assertNull(r.getLod2MultiSurface());
		assertNull(r.getLod3MultiSurface());
		assertNull(dSpace.getLod4MultiSurface());
	}

}
